"use strict";

exports.__esModule = true;
exports.useLocalStorage = useLocalStorage;

var _react = require("react");

var _reactUse = require("react-use");

function useLocalStorage(key, initialState) {
  var getLocalStorageItem = (0, _react.useCallback)(function () {
    try {
      var data = localStorage.getItem(key);

      if (data != null) {
        return JSON.parse(data);
      }

      return initialState;
    } catch (_unused) {
      return initialState;
    }
  }, [key, initialState]);

  var _useState = (0, _react.useState)(getLocalStorageItem),
      state = _useState[0],
      setState = _useState[1];

  var latestKey = (0, _reactUse.useLatest)(key);
  var latestInitialState = (0, _reactUse.useLatest)(initialState);
  var latestState = (0, _reactUse.useLatest)(state);
  (0, _reactUse.useUpdateEffect)(function () {
    setState(getLocalStorageItem());
  }, [key]);
  var set = (0, _react.useCallback)(function (nextState) {
    if (typeof nextState === 'function') {
      nextState = nextState(latestState.current);
    }

    setState(nextState);
    localStorage.setItem(latestKey.current, JSON.stringify(nextState));
  }, []);
  var reset = (0, _react.useCallback)(function () {
    localStorage.removeItem(latestKey.current);
    setState(latestInitialState.current);
  }, []);
  return [state, set, reset];
}