"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;

exports.__esModule = true;
exports.VaeNumberSchema = void 0;

var _inheritsLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/inheritsLoose"));

var _utils = require("../utils");

var _VaeLocale = require("./VaeLocale");

var _VaeSchema2 = require("./VaeSchema");

var VaeNumberSchema = /*#__PURE__*/function (_VaeSchema) {
  (0, _inheritsLoose2.default)(VaeNumberSchema, _VaeSchema);

  function VaeNumberSchema(message) {
    var _this;

    if (message === void 0) {
      message = _VaeLocale.VaeLocale.number.type;
    }

    _this = _VaeSchema.call(this, {
      type: 'number'
    }) || this;

    _this.check({
      fn: _utils.isNumeric,
      message: message
    }).transform(_utils.toNumber);

    return _this;
  }

  var _proto = VaeNumberSchema.prototype;

  _proto.min = function min(value, message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.number.min;
    }

    return this.check({
      fn: function fn(v) {
        return v >= value;
      },
      message: message,
      messageParams: {
        min: value
      },
      tag: 'min'
    });
  };

  _proto.max = function max(value, message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.number.max;
    }

    return this.check({
      fn: function fn(v) {
        return v <= value;
      },
      message: message,
      messageParams: {
        max: value
      },
      tag: 'max'
    });
  };

  _proto.lessThan = function lessThan(value, message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.number.lessThan;
    }

    return this.check({
      fn: function fn(v) {
        return v < value;
      },
      message: message,
      messageParams: {
        lessThan: value
      },
      tag: 'lessThan'
    });
  };

  _proto.moreThan = function moreThan(value, message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.number.moreThan;
    }

    return this.check({
      fn: function fn(v) {
        return v > value;
      },
      message: message,
      messageParams: {
        moreThan: value
      },
      tag: 'moreThan'
    });
  };

  _proto.integer = function integer(message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.number.integer;
    }

    return this.check({
      fn: _utils.isInteger,
      message: message,
      tag: 'integer'
    });
  };

  _proto.positive = function positive(message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.number.positive;
    }

    return this.check({
      fn: function fn(v) {
        return v > 0;
      },
      message: message,
      tag: 'positive'
    });
  };

  _proto.nonpositive = function nonpositive(message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.number.nonpositive;
    }

    return this.check({
      fn: function fn(v) {
        return v <= 0;
      },
      message: message,
      tag: 'nonpositive'
    });
  };

  _proto.negative = function negative(message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.number.negative;
    }

    return this.check({
      fn: function fn(v) {
        return v < 0;
      },
      message: message,
      tag: 'negative'
    });
  };

  _proto.nonnegative = function nonnegative(message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.number.nonnegative;
    }

    return this.check({
      fn: function fn(v) {
        return v >= 0;
      },
      message: message,
      tag: 'nonnegative'
    });
  };

  _proto.positiveInteger = function positiveInteger(message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.number.positiveInteger;
    }

    return this.check({
      fn: function fn(v) {
        return v > 0 && (0, _utils.isInteger)(v);
      },
      message: message,
      tag: 'positiveInteger'
    });
  };

  _proto.id = function id(message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.number.positiveInteger;
    }

    return this.positiveInteger(message);
  };

  return VaeNumberSchema;
}(_VaeSchema2.VaeSchema);

exports.VaeNumberSchema = VaeNumberSchema;