"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;

exports.__esModule = true;
exports.VaeStringSchema = void 0;

var _inheritsLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/inheritsLoose"));

var _utils = require("../utils");

var _VaeLocale = require("./VaeLocale");

var _VaeSchema2 = require("./VaeSchema");

var VaeStringSchema = /*#__PURE__*/function (_VaeSchema) {
  (0, _inheritsLoose2.default)(VaeStringSchema, _VaeSchema);

  function VaeStringSchema(message) {
    var _this;

    if (message === void 0) {
      message = _VaeLocale.VaeLocale.string.type;
    }

    _this = _VaeSchema.call(this, {
      type: 'string'
    }) || this;

    _this.transform(function (v) {
      return typeof v === 'number' ? String(v) : v;
    }).check({
      fn: _utils.isString,
      message: message
    });

    return _this;
  }

  var _proto = VaeStringSchema.prototype;

  _proto.min = function min(value, message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.string.min;
    }

    return this.check({
      fn: function fn(v) {
        return v.length >= value;
      },
      message: message,
      messageParams: {
        min: value
      },
      tag: 'min'
    });
  };

  _proto.max = function max(value, message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.string.max;
    }

    return this.check({
      fn: function fn(v) {
        return v.length <= value;
      },
      message: message,
      messageParams: {
        max: value
      },
      tag: 'max'
    });
  };

  _proto.length = function length(value, message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.string.length;
    }

    return this.check({
      fn: function fn(v) {
        return v.length === value;
      },
      message: message,
      messageParams: {
        length: value
      },
      tag: 'length'
    });
  };

  _proto.email = function email(message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.string.email;
    }

    return this.check({
      fn: _utils.isEmail,
      message: message,
      tag: 'email'
    });
  };

  _proto.url = function url(message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.string.url;
    }

    return this.check({
      fn: _utils.isUrl,
      message: message,
      tag: 'url'
    });
  };

  _proto.regex = function regex(value, message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.string.regex;
    }

    return this.check({
      fn: function fn(v) {
        value.lastIndex = 0;
        return value.test(v);
      },
      message: message,
      messageParams: {
        regex: value
      },
      tag: 'regex'
    });
  };

  _proto.includes = function includes(value, message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.string.includes;
    }

    return this.check({
      fn: function fn(v) {
        return (0, _utils.includes)(v, value);
      },
      message: message,
      messageParams: {
        includes: value
      },
      tag: 'includes'
    });
  };

  _proto.startsWith = function startsWith(value, message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.string.startsWith;
    }

    return this.check({
      fn: function fn(v) {
        return (0, _utils.startsWith)(v, value);
      },
      message: message,
      messageParams: {
        startsWith: value
      },
      tag: 'startsWith'
    });
  };

  _proto.endsWith = function endsWith(value, message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.string.endsWith;
    }

    return this.check({
      fn: function fn(v) {
        return (0, _utils.endsWith)(v, value);
      },
      message: message,
      messageParams: {
        endsWith: value
      },
      tag: 'endsWith'
    });
  };

  _proto.phoneNumber = function phoneNumber(message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.string.phoneNumber;
    }

    return this.check({
      fn: _utils.isPossibleChineseMobilePhoneNumber,
      message: message,
      tag: 'phoneNumber'
    });
  };

  _proto.idCardNumber = function idCardNumber(message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.string.idCardNumber;
    }

    return this.check({
      fn: _utils.isChineseIDCardNumber,
      message: message,
      tag: 'idCardNumber'
    });
  };

  _proto.cuid = function cuid(message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.string.cuid;
    }

    return this.check({
      fn: _utils.isCuid,
      message: message,
      tag: 'cuid'
    });
  };

  _proto.cuid2 = function cuid2(message) {
    if (message === void 0) {
      message = _VaeLocale.VaeLocale.string.cuid2;
    }

    return this.check({
      fn: _utils.isCuid2,
      message: message,
      tag: 'cuid2'
    });
  };

  _proto.trim = function trim() {
    this._options.stringTrim = true;
    return this;
  };

  _proto.emptyable = function emptyable() {
    this._options.stringEmptyable = true;
    return this;
  };

  return VaeStringSchema;
}(_VaeSchema2.VaeSchema);

exports.VaeStringSchema = VaeStringSchema;