import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import { isInteger, isNumeric, toNumber } from "../utils/index.js";
import { VaeLocale } from "./VaeLocale.js";
import { VaeSchema } from "./VaeSchema.js";
export var VaeNumberSchema = /*#__PURE__*/function (_VaeSchema) {
  _inheritsLoose(VaeNumberSchema, _VaeSchema);

  function VaeNumberSchema(message) {
    var _this;

    if (message === void 0) {
      message = VaeLocale.number.type;
    }

    _this = _VaeSchema.call(this, {
      type: 'number'
    }) || this;

    _this.check({
      fn: isNumeric,
      message: message
    }).transform(toNumber);

    return _this;
  }

  var _proto = VaeNumberSchema.prototype;

  _proto.min = function min(value, message) {
    if (message === void 0) {
      message = VaeLocale.number.min;
    }

    return this.check({
      fn: function fn(v) {
        return v >= value;
      },
      message: message,
      messageParams: {
        min: value
      },
      tag: 'min'
    });
  };

  _proto.max = function max(value, message) {
    if (message === void 0) {
      message = VaeLocale.number.max;
    }

    return this.check({
      fn: function fn(v) {
        return v <= value;
      },
      message: message,
      messageParams: {
        max: value
      },
      tag: 'max'
    });
  };

  _proto.lessThan = function lessThan(value, message) {
    if (message === void 0) {
      message = VaeLocale.number.lessThan;
    }

    return this.check({
      fn: function fn(v) {
        return v < value;
      },
      message: message,
      messageParams: {
        lessThan: value
      },
      tag: 'lessThan'
    });
  };

  _proto.moreThan = function moreThan(value, message) {
    if (message === void 0) {
      message = VaeLocale.number.moreThan;
    }

    return this.check({
      fn: function fn(v) {
        return v > value;
      },
      message: message,
      messageParams: {
        moreThan: value
      },
      tag: 'moreThan'
    });
  };

  _proto.integer = function integer(message) {
    if (message === void 0) {
      message = VaeLocale.number.integer;
    }

    return this.check({
      fn: isInteger,
      message: message,
      tag: 'integer'
    });
  };

  _proto.positive = function positive(message) {
    if (message === void 0) {
      message = VaeLocale.number.positive;
    }

    return this.check({
      fn: function fn(v) {
        return v > 0;
      },
      message: message,
      tag: 'positive'
    });
  };

  _proto.nonpositive = function nonpositive(message) {
    if (message === void 0) {
      message = VaeLocale.number.nonpositive;
    }

    return this.check({
      fn: function fn(v) {
        return v <= 0;
      },
      message: message,
      tag: 'nonpositive'
    });
  };

  _proto.negative = function negative(message) {
    if (message === void 0) {
      message = VaeLocale.number.negative;
    }

    return this.check({
      fn: function fn(v) {
        return v < 0;
      },
      message: message,
      tag: 'negative'
    });
  };

  _proto.nonnegative = function nonnegative(message) {
    if (message === void 0) {
      message = VaeLocale.number.nonnegative;
    }

    return this.check({
      fn: function fn(v) {
        return v >= 0;
      },
      message: message,
      tag: 'nonnegative'
    });
  };

  _proto.positiveInteger = function positiveInteger(message) {
    if (message === void 0) {
      message = VaeLocale.number.positiveInteger;
    }

    return this.check({
      fn: function fn(v) {
        return v > 0 && isInteger(v);
      },
      message: message,
      tag: 'positiveInteger'
    });
  };

  _proto.id = function id(message) {
    if (message === void 0) {
      message = VaeLocale.number.positiveInteger;
    }

    return this.positiveInteger(message);
  };

  return VaeNumberSchema;
}(VaeSchema);