/**
 * vae 数据验证。
 *
 * @packageDocumentation
 */

declare function array<T extends Nullable<unknown[]> = unknown[]>(element: VaeArraySchemaElementOf<T>, message?: VaeLocaleMessage): VaeArraySchema<T>;

declare function array<T extends Nullable<unknown[]> = unknown[]>(schema: (schema: VaeArraySchema<T>) => VaeArraySchema<T>, message?: VaeLocaleMessage): VaeArraySchema<T>;

declare function array<T extends Nullable<unknown[]> = unknown[]>(message?: VaeLocaleMessage): VaeArraySchema<T>;

declare function boolean<T extends Nullable<boolean> = boolean>(schema: (schema: VaeBooleanSchema<T>) => VaeBooleanSchema<T>, message?: VaeLocaleMessage): VaeBooleanSchema<T>;

declare function boolean<T extends Nullable<boolean> = boolean>(message?: VaeLocaleMessage): VaeBooleanSchema<T>;

declare function date<T extends Nullable<Date> = Date>(schema: (schema: VaeDateSchema<T>) => VaeDateSchema<T>, message?: VaeLocaleMessage): VaeDateSchema<T>;

declare function date<T extends Nullable<Date> = Date>(message?: VaeLocaleMessage): VaeDateSchema<T>;

declare type DotPath<T> = object extends T ? string : T extends any[] ? '0' | SubKeys<T, 0> : T extends readonly any[] ? Extract<keyof T, `${number}`> | SubKeys<T, Extract<keyof T, `${number}`>> : T extends object ? Extract<keyof T, string> | SubKeys<T, Extract<keyof T, string>> : never;

declare type DotPathValue<T, Path extends string> = Path extends '0' ? T[0] : Path extends keyof T ? T[Path] : Path extends `${infer K}.${infer R}` ? K extends '0' ? DotPathValue<T[0], R> : K extends keyof T ? DotPathValue<T[K], R> : unknown : unknown;

/**
 * 判断 `T` 是否是 `any` 类型。
 *
 * @public
 * @see https://stackoverflow.com/a/49928360
 * @example
 * ```typescript
 * type X = IsAny<any>
 * // => true
 * ```
 */
declare type IsAny<T> = 0 extends 1 & T ? true : false;

declare type Nullable<T> = T | null | undefined;

declare function number<T extends Nullable<number> = number>(schema: (schema: VaeNumberSchema<T>) => VaeNumberSchema<T>, message?: VaeLocaleMessage): VaeNumberSchema<T>;

declare function number<T extends Nullable<number> = number>(message?: VaeLocaleMessage): VaeNumberSchema<T>;

declare function object<T0 extends Record<any, Nullable<any>> = Record<any, any>, T extends NonNullable<T0> = NonNullable<T0>>(shape: VaeObjectSchemaShapeOf<T>, message?: VaeLocaleMessage): VaeObjectSchema<IsAny<keyof T0> extends true ? T : T0>;

declare function object<T extends Nullable<Record<any, any>> = Record<any, any>>(schema: (schema: VaeObjectSchema<T>) => VaeObjectSchema<T>, message?: VaeLocaleMessage): VaeObjectSchema<T>;

declare function object<T extends Nullable<Record<any, any>> = Record<any, any>>(message?: VaeLocaleMessage): VaeObjectSchema<T>;

/** Mark some properties as optional, leaving others unchanged */
declare type PartialBy<T, K extends keyof T> = Omit<T, K> & Partial<Pick<T, K>>;

/** Mark some properties as required, leaving others unchanged */
declare type RequiredBy<T, RK extends keyof T> = Exclude<T, RK> & Required<Pick<T, RK>>;

declare const schemaBuilders: {
    string: typeof string;
    number: typeof number;
    date: typeof date;
    boolean: typeof boolean;
    object: typeof object;
    array: typeof array;
};

declare function string<T extends Nullable<string> = string>(schema: (schema: VaeStringSchema<T>) => VaeStringSchema<T>, message?: VaeLocaleMessage): VaeStringSchema<T>;

declare function string<T extends Nullable<string> = string>(message?: VaeLocaleMessage): VaeStringSchema<T>;

declare type SubKeys<T, K extends string | number> = K extends keyof T ? `${K}.${DotPath<T[K]>}` : never;

export declare const v: {
    create: <R, F extends (_: typeof schemaBuilders) => R = (_: typeof schemaBuilders) => R>(cb: F) => R;
    localeBuilder: typeof VaeLocaleBuilder;
    setLocale: (locale: VaeLocaleShape) => void;
    string: typeof string;
    number: typeof number;
    date: typeof date;
    boolean: typeof boolean;
    object: typeof object;
    array: typeof array;
};

export declare class VaeArraySchema<T0 extends Nullable<any[]> = any[], T extends NonNullable<T0> = NonNullable<T0>> extends VaeSchema<T0> {
    constructor(element?: VaeArraySchemaElementOf<T>, message?: VaeLocaleMessage);
    element(element: VaeArraySchemaElementOf<T>): this;
    nonempty(message?: VaeLocaleMessage): this;
    min(value: number, message?: VaeLocaleMessage): this;
    max(value: number, message?: VaeLocaleMessage): this;
    length(value: number, message?: VaeLocaleMessage): this;
}

export declare type VaeArraySchemaElementOf<T> = T extends Array<infer X> ? VaeSchemaOf<X> : never;

export declare class VaeBooleanSchema<T0 extends Nullable<boolean> = boolean> extends VaeSchema<T0> {
    constructor(message?: VaeLocaleMessage);
    true(message?: VaeLocaleMessage): this;
    false(message?: VaeLocaleMessage): this;
}

export declare class VaeDateSchema<T0 extends Nullable<Date> = Date> extends VaeSchema<T0> {
    constructor(message?: VaeLocaleMessage);
    min(value: Date | string | number, message?: VaeLocaleMessage): this;
    max(value: Date | string | number, message?: VaeLocaleMessage): this;
}

export declare class VaeError extends Error {
    issues: VaeIssue[];
    name: string;
    constructor(issues: VaeIssue[]);
    static messageFromIssues(issues: VaeIssue[]): string;
}

export declare interface VaeErrorIssue {
    path: Array<string | number>;
    message: string;
}

export declare type VaeIssue = {
    path: VaeSchemaPath;
    message: string;
};

export declare const VaeLocale: {
    $set: (locale: VaeLocaleShape) => void;
    base: Record<'required' | 'enum' | 'custom', VaeLocaleMessage>;
    string: Record<'type' | 'min' | 'max' | 'length' | 'email' | 'url' | 'regex' | 'includes' | 'startsWith' | 'endsWith' | 'phoneNumber' | 'idCardNumber' | 'cuid' | 'cuid2', VaeLocaleMessage>;
    object: Record<'type' | 'requiredFieldsAtLeastOne', VaeLocaleMessage>;
    number: Record<'type' | 'min' | 'max' | 'lessThan' | 'moreThan' | 'integer' | 'positive' | 'nonpositive' | 'negative' | 'nonnegative' | 'positiveInteger', VaeLocaleMessage>;
    boolean: Record<'type' | 'true' | 'false', VaeLocaleMessage>;
    array: Record<'type' | 'nonempty' | 'min' | 'max' | 'length', VaeLocaleMessage>;
    date: Record<'type' | 'min' | 'max', VaeLocaleMessage>;
};

export declare class VaeLocaleBuilder {
    static zhCN(options: {
        getLabel: (payload: VaeLocaleMessagePayload) => string;
    }): VaeLocaleShape;
}

export declare type VaeLocaleMessage = string | ((payload: VaeLocaleMessagePayload) => string);

export declare type VaeLocaleMessagePayload = {
    label?: string;
    path: VaeSchemaPath;
    value: any;
    params: Record<string, any>;
};

export declare type VaeLocaleShape = {
    base: Record<'required' | 'enum' | 'custom', VaeLocaleMessage>;
    string: Record<'type' | 'min' | 'max' | 'length' | 'email' | 'url' | 'regex' | 'includes' | 'startsWith' | 'endsWith' | 'phoneNumber' | 'idCardNumber' | 'cuid' | 'cuid2', VaeLocaleMessage>;
    object: Record<'type' | 'requiredFieldsAtLeastOne', VaeLocaleMessage>;
    number: Record<'type' | 'min' | 'max' | 'lessThan' | 'moreThan' | 'integer' | 'positive' | 'nonpositive' | 'negative' | 'nonnegative' | 'positiveInteger', VaeLocaleMessage>;
    boolean: Record<'type' | 'true' | 'false', VaeLocaleMessage>;
    array: Record<'type' | 'nonempty' | 'min' | 'max' | 'length', VaeLocaleMessage>;
    date: Record<'type' | 'min' | 'max', VaeLocaleMessage>;
};

export declare class VaeNumberSchema<T0 extends Nullable<number> = number> extends VaeSchema<T0> {
    constructor(message?: VaeLocaleMessage);
    min(value: number, message?: VaeLocaleMessage): this;
    max(value: number, message?: VaeLocaleMessage): this;
    lessThan(value: number, message?: VaeLocaleMessage): this;
    moreThan(value: number, message?: VaeLocaleMessage): this;
    integer(message?: VaeLocaleMessage): this;
    positive(message?: VaeLocaleMessage): this;
    nonpositive(message?: VaeLocaleMessage): this;
    negative(message?: VaeLocaleMessage): this;
    nonnegative(message?: VaeLocaleMessage): this;
    positiveInteger(message?: VaeLocaleMessage): this;
    id(message?: VaeLocaleMessage): this;
}

export declare class VaeObjectSchema<T0 extends Nullable<Record<any, any>> = Record<any, any>, T extends NonNullable<T0> = NonNullable<T0>> extends VaeSchema<T0> {
    constructor(shape?: VaeObjectSchemaShapeOf<T>, message?: VaeLocaleMessage);
    shape(shape: VaeObjectSchemaShapeOf<T>): this;
    pickFields<K extends keyof T>(keys: K[]): VaeObjectSchema<Pick<T, K>>;
    omitFields<K extends keyof T>(keys: K[]): VaeObjectSchema<Omit<T, K>>;
    optionalFields<K extends keyof T>(keys: K[]): VaeObjectSchema<PartialBy<T, K>>;
    optionalFields(): VaeObjectSchema<Partial<T>>;
    requiredFields<K extends keyof T>(keys: K[]): VaeObjectSchema<RequiredBy<T, K>>;
    requiredFields(): VaeObjectSchema<Required<T>>;
    shapeOfFields<K extends keyof T>(keys: K[]): VaeObjectSchemaShapeOf<Pick<T, K>>;
    shapeOfFields(): VaeObjectSchemaShapeOf<T>;
    requiredFieldsAtLeastOne<K extends keyof T>(keys: K[], message?: VaeLocaleMessage): this;
}

export declare type VaeObjectSchemaShapeOf<T> = {
    [K in keyof T]: VaeSchemaOf<T[K]>;
};

export declare abstract class VaeSchema<T0 extends any = any, T extends NonNullable<T0> = NonNullable<T0>> {
    protected _options: RequiredBy<VaeSchemaOptions<T, any>, 'processors'>;
    constructor(options: VaeSchemaOptions<T, any>);
    get options(): RequiredBy<VaeSchemaOptions<T, any>, "processors">;
    meta(metadata: VaeSchemaMetadata): this;
    check(payload: VaeSchemaCheckPayload<T>): this;
    transform(payload: VaeSchemaTransformPayload<T>): this;
    label(label: string): this;
    default(value: T | (() => T)): this;
    required(message?: VaeLocaleMessage): this;
    optional(): this;
    enum(value: T[] | Record<any, T>, message?: VaeLocaleMessage): this;
    custom(fn: (value: T) => boolean | void, messageOrOptions?: VaeLocaleMessage | {
        message?: VaeLocaleMessage;
        path?: DotPath<T>;
        tag?: string;
    }): this;
    runtime(fn: VaeSchemaRuntimeFn<T, this>): this;
    clone(): this;
    reach<P extends DotPath<T>>(paths: P[], ctx?: VaeSchemaReachContext, curPath?: VaeSchemaPath): {
        [K in P]: VaeSchemaOf<DotPathValue<T, K>>;
    };
    reach<P extends DotPath<T>>(path: P, ctx?: VaeSchemaReachContext, curPath?: VaeSchemaPath): VaeSchemaOf<DotPathValue<T, P>>;
    parse(data: T0, options?: VaeSchemaParseOptions): VaeSchemaParseResult<T0>;
    parseOrThrow(data: T0, options?: VaeSchemaParseOptions): T0;
    cast(data: any, options?: VaeSchemaCastOptions): any;
}

export declare type VaeSchemaCastOptions = {
    /**
     * 是否保留未知属性
     *
     * @default false
     */
    preserveUnknownKeys?: boolean;
};

export declare type VaeSchemaCheckPayload<T> = {
    fn: ((value: T) => boolean | void) | VaeSchema;
    message: VaeLocaleMessage;
    messageParams?: Record<string, any>;
    path?: VaeSchemaPath;
    tag?: string;
};

export declare interface VaeSchemaCustomMetadata {
}

export declare type VaeSchemaMetadata = keyof VaeSchemaCustomMetadata extends never ? Record<any, any> : VaeSchemaCustomMetadata;

export declare type VaeSchemaOf<T0, T extends NonNullable<T0> = NonNullable<T0>> = [
T
] extends [string] ? VaeStringSchema<T0> : [T] extends [number] ? VaeNumberSchema<T0> : [T] extends [boolean] ? VaeBooleanSchema<T0> : [T] extends [Date] ? VaeDateSchema<T0> : T extends any[] ? VaeArraySchema<T0> : VaeObjectSchema<T0>;

export declare type VaeSchemaOptions<T, S> = {
    type: VaeSchemaType;
    label?: string;
    default?: T | (() => T);
    required?: boolean;
    requiredMessage?: VaeLocaleMessage;
    objectKeys?: string[];
    stringTrim?: boolean;
    stringEmptyable?: boolean;
    processors?: Array<VaeSchemaCheckPayload<T> | VaeSchemaTransformPayload<T>>;
    runtime?: VaeSchemaRuntimeFn<T, S>;
    metadata?: VaeSchemaMetadata;
};

export declare class VaeSchemaParseContext {
    issues: VaeIssue[];
    addIssue(issue: VaeIssue): void;
}

export declare type VaeSchemaParseOptions = {
    /**
     * 上下文，内部使用
     *
     * @inner
     */
    ctx?: VaeSchemaParseContext;
    /**
     * 当前路径，内部使用
     *
     * @inner
     */
    curPath?: VaeSchemaPath;
    /**
     * 是否提前终止
     *
     * @default false
     */
    abortEarly?: boolean;
    /**
     * 是否保留未知属性
     *
     * @default false
     */
    preserveUnknownKeys?: boolean;
    /**
     * 是否 cast 模式
     *
     * @default false
     */
    cast?: boolean;
};

export declare type VaeSchemaParseResult<T> = {
    success: true;
    data: T;
} | {
    success: false;
    issues: VaeIssue[];
    message: string;
};

export declare type VaeSchemaPath = Array<string | number>;

export declare class VaeSchemaReachContext {
    schemas: Array<{
        path: string;
        schema: VaeSchema;
    }>;
    addSchema(path: string, schema: VaeSchema): void;
}

export declare type VaeSchemaRuntimeFn<T, S> = (payload: {
    value: T;
    schema: S;
}) => any;

export declare type VaeSchemaTransformPayload<T> = (value: T) => T;

export declare type VaeSchemaType = 'string' | 'number' | 'object' | 'date' | 'boolean' | 'array';

export declare class VaeStringSchema<T0 extends Nullable<string> = string> extends VaeSchema<T0> {
    constructor(message?: VaeLocaleMessage);
    min(value: number, message?: VaeLocaleMessage): this;
    max(value: number, message?: VaeLocaleMessage): this;
    length(value: number, message?: VaeLocaleMessage): this;
    email(message?: VaeLocaleMessage): this;
    url(message?: VaeLocaleMessage): this;
    regex(value: RegExp, message?: VaeLocaleMessage): this;
    includes(value: string, message?: VaeLocaleMessage): this;
    startsWith(value: string, message?: VaeLocaleMessage): this;
    endsWith(value: string, message?: VaeLocaleMessage): this;
    phoneNumber(message?: VaeLocaleMessage): this;
    idCardNumber(message?: VaeLocaleMessage): this;
    cuid(message?: VaeLocaleMessage): this;
    cuid2(message?: VaeLocaleMessage): this;
    trim(): this;
    emptyable(): this;
}

export { }
