import _extends from "@babel/runtime/helpers/esm/extends";
import { VaeArraySchema } from "./VaeArraySchema.js";
import { VaeBooleanSchema } from "./VaeBooleanSchema.js";
import { VaeDateSchema } from "./VaeDateSchema.js";
import { VaeLocale, VaeLocaleBuilder } from "./VaeLocale.js";
import { VaeNumberSchema } from "./VaeNumberSchema.js";
import { VaeObjectSchema } from "./VaeObjectSchema.js";
import { VaeSchema } from "./VaeSchema.js";
import { VaeStringSchema } from "./VaeStringSchema.js";

function string(messageOrSchema, message) {
  return typeof messageOrSchema === 'function' ? messageOrSchema(new VaeStringSchema(message)) : new VaeStringSchema(messageOrSchema);
}

function number(messageOrSchema, message) {
  return typeof messageOrSchema === 'function' ? messageOrSchema(new VaeNumberSchema(message)) : new VaeNumberSchema(messageOrSchema);
}

function date(messageOrSchema, message) {
  return typeof messageOrSchema === 'function' ? messageOrSchema(new VaeDateSchema(message)) : new VaeDateSchema(messageOrSchema);
}

function _boolean(messageOrSchema, message) {
  return typeof messageOrSchema === 'function' ? messageOrSchema(new VaeBooleanSchema(message)) : new VaeBooleanSchema(messageOrSchema);
}

function object(messageOrSchemaOrShape, message) {
  return typeof messageOrSchemaOrShape === 'function' ? messageOrSchemaOrShape(new VaeObjectSchema(undefined, message)) : typeof messageOrSchemaOrShape === 'object' ? new VaeObjectSchema(messageOrSchemaOrShape, message) : new VaeObjectSchema(undefined, messageOrSchemaOrShape);
}

function array(messageOrSchemaOrElement, message) {
  return typeof messageOrSchemaOrElement === 'function' ? messageOrSchemaOrElement(new VaeArraySchema(undefined, message)) : messageOrSchemaOrElement instanceof VaeSchema ? new VaeArraySchema(messageOrSchemaOrElement, message) : new VaeArraySchema(undefined, messageOrSchemaOrElement);
}

var schemaBuilders = {
  string: string,
  number: number,
  date: date,
  boolean: _boolean,
  object: object,
  array: array
};
export var v = _extends({}, schemaBuilders, {
  create: function create(cb) {
    return cb(schemaBuilders);
  },
  localeBuilder: VaeLocaleBuilder,
  setLocale: function setLocale(locale) {
    VaeLocale.$set(locale);
  }
}); // @index('./Vae*.ts', f => `export * from '${f.path}'`)

export * from "./VaeArraySchema.js";
export * from "./VaeBooleanSchema.js";
export * from "./VaeDateSchema.js";
export * from "./VaeError.js";
export * from "./VaeIssue.js";
export * from "./VaeLocale.js";
export * from "./VaeNumberSchema.js";
export * from "./VaeObjectSchema.js";
export * from "./VaeSchema.js";
export * from "./VaeSchemaParseContext.js";
export * from "./VaeSchemaReachContext.js";
export * from "./VaeStringSchema.js"; // @endindex